#!/usr/bin/env python3
from pathlib import Path
from typing import List, Sequence

from ltpylib import files, filters


FIND_REPOS_RECURSION_EXCLUDES = frozenset([
  'node_modules'
])


def filter_invalid_repos(git_repos: List[Path]) -> List[Path]:
  filtered = []
  for repo in git_repos:
    if not repo.is_dir():
      continue

    if not repo.joinpath('.git').is_dir():
      continue

    filtered.append(repo)

  return filtered


def print_repos(git_repos: List[Path]):
  for repo in git_repos:
    print(repo.as_posix())


def find_git_repos(
    base_dir: Path,
    max_depth: int = -1,
    recursion_include_patterns: Sequence[str] = None,
    recursion_exclude_patterns: Sequence[str] = None,
    recursion_includes: Sequence[str] = None,
    recursion_excludes: Sequence[str] = FIND_REPOS_RECURSION_EXCLUDES
) -> List[Path]:
  dotgit_dirs = files.find_children(
    base_dir,
    break_after_match=True,
    include_files=False,
    max_depth=max_depth,
    includes=['.git'],
    recursion_include_patterns=recursion_include_patterns,
    recursion_exclude_patterns=recursion_exclude_patterns,
    recursion_includes=recursion_includes,
    recursion_excludes=recursion_excludes
  )
  return [
    dotgit.parent for dotgit in dotgit_dirs
  ]


def add_git_dirs(
    git_repos: List[Path],
    add_dir: List[Path],
    exclude_patterns: List[str] = None,
    max_depth: int = -1,
    recursion_include_patterns: Sequence[str] = None,
    recursion_exclude_patterns: Sequence[str] = None,
    recursion_includes: Sequence[str] = None,
    recursion_excludes: Sequence[str] = FIND_REPOS_RECURSION_EXCLUDES
) -> List[Path]:
  for git_dir in add_dir:
    if not git_dir.is_dir():
      continue

    add_dir_repos = find_git_repos(
      git_dir,
      max_depth=max_depth,
      recursion_include_patterns=recursion_include_patterns,
      recursion_exclude_patterns=recursion_exclude_patterns,
      recursion_includes=recursion_includes,
      recursion_excludes=recursion_excludes
    )
    add_dir_repos.sort()
    for git_repo in add_dir_repos:
      if should_skip_repo(git_repo, exclude_patterns):
        continue

      git_repos.append(git_repo)

  return git_repos


def should_skip_repo(git_repo: Path, exclude_patterns: List[str]) -> bool:
  return filters.should_skip(git_repo, exclude_patterns=exclude_patterns)



#!/usr/bin/env python3
from pathlib import Path
from typing import List, Tuple, Union

from ltpylib import procs, xmlparser

MVN_EXPR_PROJECT_GROUP_ID = 'project.groupId'
MVN_EXPR_PROJECT_ARTIFACT_ID = 'project.artifactId'
MVN_EXPR_PROJECT_VERSION = 'project.version'


def get_project_artifact_id(pom: Union[str, Path], use_mvn_expression: bool = False) -> str:
  result: str = None

  if not use_mvn_expression:
    result = xmlparser.parse_xml(pom).findtext('artifactId')

  if use_mvn_expression or not result:
    result = run_mvn_expression(pom, MVN_EXPR_PROJECT_ARTIFACT_ID)

  return result


def get_project_group_id(pom: Union[str, Path], use_mvn_expression: bool = False) -> str:
  result: str = None

  if not use_mvn_expression:
    parsed = xmlparser.parse_xml(pom)
    result = parsed.findtext('groupId')
    if not result:
      result = parsed.findtext('./parent/groupId')

  if use_mvn_expression or not result:
    result = run_mvn_expression(pom, MVN_EXPR_PROJECT_GROUP_ID)

  return result


def get_project_version(pom: Union[str, Path], use_mvn_expression: bool = False) -> str:
  result: str = None

  if not use_mvn_expression:
    parsed = xmlparser.parse_xml(pom)
    result = parsed.findtext('version')
    if not result:
      result = parsed.findtext('./parent/version')

  if use_mvn_expression or not result:
    result = run_mvn_expression(pom, MVN_EXPR_PROJECT_VERSION)

  return result


def parse_mvn_expression_output(mvn_out: str) -> str:
  return [line for line in mvn_out.splitlines() if not '[INFO]' in line][-1]


def run_mvn_expression(pom: Union[str, Path], expression: str) -> str:
  if isinstance(pom, str):
    pom = Path(pom)

  mvn_out = procs.run_and_parse_output(
    ['mvn', '-f', pom.absolute().as_posix(), 'help:evaluate', '-Dexpression={0}'.format(expression), '--quiet', '-DforceStdout'],
    check=True,
  )[1].strip()
  return mvn_out


def update_dep_version(pom: Union[str, Path], dep: str, version: str, additional_arguments: List[str] = None, check: bool = False) -> Tuple[int, str]:
  if isinstance(pom, str):
    pom = Path(pom)

  if dep.count(':') <= 1:
    dep += ':*'

  mvn_command = [
    'mvn',
    '-f',
    pom.absolute().as_posix(),
    'versions:use-dep-version',
    '-DdepVersion=%s' % version,
    '-DgenerateBackupPoms=false',
    '-Dincludes=%s' % dep,
    '-DallowSnapshots=true',
  ]
  if additional_arguments:
    mvn_command.extend(additional_arguments)

  return procs.run_and_parse_output(mvn_command, check=check)


def update_project_version(pom: Union[str, Path], version: str, additional_arguments: List[str] = None, check: bool = False) -> Tuple[int, str]:
  if isinstance(pom, str):
    pom = Path(pom)

  mvn_command = [
    'mvn',
    '-f',
    pom.absolute().as_posix(),
    'versions:set',
    '-DgenerateBackupPoms=false',
    '-DnewVersion=' + version,
  ]
  if additional_arguments:
    mvn_command.extend(additional_arguments)

  return procs.run_and_parse_output(mvn_command, check=check)


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
# pylint: disable=C0111
import logging
import re
from pathlib import Path
from typing import Sequence, Union

from ltpylib import procs


def should_include(
    test_value: Union[str, Path],
    include_patterns: Sequence[str] = None,
    exclude_patterns: Sequence[str] = None,
    includes: Sequence[str] = None,
    excludes: Sequence[str] = None,
    verbose: bool = False
) -> bool:
  return not should_skip(
    test_value,
    include_patterns=include_patterns,
    exclude_patterns=exclude_patterns,
    includes=includes,
    excludes=excludes,
    verbose=verbose
  )


def should_skip(
    test_value: Union[str, Path],
    include_patterns: Sequence[str] = None,
    exclude_patterns: Sequence[str] = None,
    includes: Sequence[str] = None,
    excludes: Sequence[str] = None,
    verbose: bool = False
) -> bool:
  if not include_patterns and not exclude_patterns and not includes and not excludes:
    return False

  if isinstance(test_value, str):
    test_str = str(test_value)
  else:
    test_str = test_value.as_posix()

  if excludes and test_str in excludes:
    if verbose:
      logging.info('Excluded: %s', test_str)
    return True

  if includes:
    if test_str in includes:
      if verbose:
        logging.info('Included: %s', test_str)
      return False
    elif not include_patterns:
      return True

  if exclude_patterns:
    for regex in exclude_patterns:
      if re.search(regex, test_str):
        if verbose:
          logging.info('Excluded by pattern: pattern=%s path=%s', regex, test_str)
        return True

  if include_patterns:
    for regex in include_patterns:
      if re.search(regex, test_str):
        if verbose:
          logging.info('Included by pattern: pattern=%s path=%s', regex, test_str)
        return False

    return True

  return False


def should_skip_from_cmds(
    test_value: Union[str, Path],
    include_commands: Sequence[str] = None,
    exclude_commands: Sequence[str] = None,
    verbose: bool = False
) -> bool:
  if not include_commands and not exclude_commands:
    return False

  if isinstance(test_value, str):
    test_str = str(test_value)
  else:
    test_str = test_value.as_posix()

  if exclude_commands:
    for command in exclude_commands:
      result = procs.run(
        command,
        cwd=test_str,
        universal_newlines=True,
        shell=True,
        executable='/usr/local/bin/bash',
      )
      if result.returncode == 0:
        if verbose:
          logging.info('Skipped path: path=%s command=%s', test_str, command)
        return True

  if include_commands:
    for command in include_commands:
      result = procs.run(
        command,
        cwd=test_str,
        universal_newlines=True,
        shell=True,
        executable='/usr/local/bin/bash',
      )
      if result.returncode == 0:
        if verbose:
          logging.info('Included path: path=%s command=%s', test_str, command)
        return False

    return True

  return False


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
# pylint: disable=C0111
import re
from typing import Union


def replace_matches(content: str, search_string: str, replacement: str, quote_replacement: Union[bool, str] = False) -> str:
  if isinstance(quote_replacement, str):
    quote_replacement = quote_replacement.lower() in ['true', '1', 't', 'y', 'yes']

  if quote_replacement:
    replacement = re.escape(replacement)

  return re.sub(search_string, replacement, content)


if __name__ == "__main__":
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3
from typing import List


class DataWithUnknownProperties(object):

  def __init__(self, values: dict = None):
    if values:
      self.unknownProperties: dict = values


class DisplayIdAndId(object):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.displayId: str = values.pop("displayId", None)
    self.id: str = values.pop("id", None)


class ApplicationProperties(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.buildDate: str = values.pop("buildDate", None)
    self.buildNumber: str = values.pop("buildNumber", None)
    self.displayName: str = values.pop("displayName", None)
    self.version: str = values.pop("version", None)

    DataWithUnknownProperties.__init__(self, values)


class Branch(DataWithUnknownProperties, DisplayIdAndId):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    DisplayIdAndId.__init__(self, values)

    self.isDefault: bool = values.pop("isDefault", None)
    self.latestChangeset: str = values.pop("latestChangeset", None)

    DataWithUnknownProperties.__init__(self, values)


class Comment(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.id: int = values.pop("id", None)
    self.text: str = values.pop("text", None)
    self.version: int = values.pop("version", None)

    DataWithUnknownProperties.__init__(self, values)


class Commit(DataWithUnknownProperties, DisplayIdAndId):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    DisplayIdAndId.__init__(self, values)

    DataWithUnknownProperties.__init__(self, values)


class Link(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.href: str = values.pop("href", None)
    self.name: str = values.pop("name", None)

    DataWithUnknownProperties.__init__(self, values)


class Links(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.clone: List[Link] = list(map(Link, values.pop("clone", []))) if "clone" in values else None
    self.self: List[Link] = list(map(Link, values.pop("self", []))) if "self" in values else None

    DataWithUnknownProperties.__init__(self, values)


class Project(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.description: str = values.pop("description", None)
    self.id: int = values.pop("id", None)
    self.key: str = values.pop("key", None)
    self.links: Links = Links(values=values.pop("links")) if "links" in values else None
    self.name: str = values.pop("name", None)
    self.owner: User = User(values=values.pop("owner")) if "owner" in values else None
    self.public: bool = values.pop("public", None)
    self.type: str = values.pop("type", None)

    DataWithUnknownProperties.__init__(self, values)


class PullRequestMergeability(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.canMerge: bool = values.pop("canMerge", None)
    self.conflicted: bool = values.pop("conflicted", None)
    self.outcome: str = values.pop("outcome", None)
    self.vetoes: List[PullRequestVeto] = list(map(PullRequestVeto, values.pop("vetoes", []))) if "vetoes" in values else None

    DataWithUnknownProperties.__init__(self, values)


class PullRequestParticipant(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.approved: bool = values.pop("approved", None)
    self.lastReviewedCommit: str = values.pop("lastReviewedCommit", None)
    self.role: str = values.pop("role", None)
    self.status: str = values.pop("status", None)
    self.user: User = User(values=values.pop("user")) if "user" in values else None

    DataWithUnknownProperties.__init__(self, values)


class PullRequestProperties(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.commentCount: int = values.pop("commentCount", None)
    self.mergeCommit: Commit = Commit(values=values.pop("mergeCommit")) if "mergeCommit" in values else None
    self.openTaskCount: int = values.pop("openTaskCount", None)
    self.resolvedTaskCount: int = values.pop("resolvedTaskCount", None)

    DataWithUnknownProperties.__init__(self, values)


class PullRequestRef(DataWithUnknownProperties, DisplayIdAndId):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    DisplayIdAndId.__init__(self, values)

    self.latestCommit: str = values.pop("latestCommit", None)
    self.repository: Repository = Repository(values=values.pop("repository")) if "repository" in values else None

    DataWithUnknownProperties.__init__(self, values)


class PullRequestStatus(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.author: PullRequestParticipant = PullRequestParticipant(values=values.pop("author")) if "author" in values else None
    self.closed: bool = values.pop("closed", None)
    self.createdDate: int = values.pop("createdDate", None)
    self.description: str = values.pop("description", None)
    self.fromRef: PullRequestRef = PullRequestRef(values=values.pop("fromRef")) if "fromRef" in values else None
    self.id: int = values.pop("id", None)
    self.links: Links = Links(values=values.pop("links")) if "links" in values else None
    self.locked: bool = values.pop("locked", None)
    self.open: bool = values.pop("open", None)
    self.outstandingTaskCount: int = values.pop("outstandingTaskCount", None)
    self.participants: List[PullRequestParticipant] = list(map(PullRequestParticipant, values.pop("participants", []))) if "participants" in values else None
    self.properties: PullRequestProperties = PullRequestProperties(values=values.pop("properties")) if "properties" in values else None
    self.reviewers: List[PullRequestParticipant] = list(map(PullRequestParticipant, values.pop("reviewers", []))) if "reviewers" in values else None
    self.state: str = values.pop("state", None)
    self.title: str = values.pop("title", None)
    self.toRef: PullRequestRef = PullRequestRef(values=values.pop("toRef")) if "toRef" in values else None
    self.updatedDate: int = values.pop("updatedDate", None)
    self.version: int = values.pop("version", None)
    self.mergeInfo: PullRequestMergeability = None

    DataWithUnknownProperties.__init__(self, values)


class PullRequestMergeStatus(PullRequestStatus):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.sourceBranchDeleted: bool = None

    PullRequestStatus.__init__(self, values)


class PullRequestVeto(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.detailedMessage: str = values.pop("detailedMessage", None)
    self.summaryMessage: str = values.pop("summaryMessage", None)

    DataWithUnknownProperties.__init__(self, values)


class Repository(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.forkable: bool = values.pop("forkable", None)
    self.id: int = values.pop("id", None)
    self.links: Links = Links(values=values.pop("links")) if "links" in values else None
    self.name: str = values.pop("name", None)
    self.origin: Repository = Repository(values=values.pop("origin")) if "origin" in values else None
    self.project: Project = Project(values=values.pop("project")) if "project" in values else None
    self.public: bool = values.pop("public", None)
    self.scmId: str = values.pop("scmId", None)
    self.slug: str = values.pop("slug", None)
    self.state: str = values.pop("state", None)
    self.statusMessage: str = values.pop("statusMessage", None)

    DataWithUnknownProperties.__init__(self, values)


class Tag(DataWithUnknownProperties, DisplayIdAndId):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    DisplayIdAndId.__init__(self, values)

    self.hash: str = values.pop("hash", None)
    self.latestChangeset: str = values.pop("latestChangeset", None)
    self.latestCommit: str = values.pop("latestCommit", None)

    DataWithUnknownProperties.__init__(self, values)


class User(DataWithUnknownProperties):

  def __init__(self, values: dict = None):
    values = values if values is not None else {}

    self.active: bool = values.pop("active", None)
    self.deletable: bool = values.pop("deletable", None)
    self.displayName: str = values.pop("displayName", None)
    self.emailAddress: str = values.pop("emailAddress", None)
    self.id: int = values.pop("id", None)
    self.lastAuthenticationTimestamp: int = values.pop("lastAuthenticationTimestamp", None)
    self.links: Links = Links(values=values.pop("links")) if "links" in values else None
    self.mutableDetails: bool = values.pop("mutableDetails", None)
    self.mutableGroups: bool = values.pop("mutableGroups", None)
    self.name: str = values.pop("name", None)
    self.slug: str = values.pop("slug", None)
    self.type: str = values.pop("type", None)

    DataWithUnknownProperties.__init__(self, values)



#!/usr/bin/env python3

import glob
import os
from collections import OrderedDict

try:
  import configparser as cp
except:
  import ConfigParser as cp

try:
  import io as io_compat
except:
  import StringIO as io_compat


def get_host(key, use_key_as_default=True):
  hosts_dir = os.path.abspath('%s/config/hosts' % os.getenv('DOTFILES', '%s/.dotfiles' % os.getenv('HOME')))
  hosts = cp.ConfigParser()
  host_files = glob.glob(hosts_dir + '/*.properties')
  host_files.extend(glob.glob(hosts_dir + '/*/*.properties'))
  for hosts_file in host_files:
    hosts = _py2_read_properties(hosts_file, config=hosts)

  if use_key_as_default:
    try:
      return hosts.get('DEFAULT', key)
    except cp.NoOptionError:
      return key
    except cp.NoSectionError:
      return key
  else:
    return hosts.get('DEFAULT', key)


def config_parser_to_string(config, sort_keys=False):
  if sort_keys:
    if config._defaults:
      config._defaults = OrderedDict(sorted(config._defaults.items(), key=lambda t: t[0]))

    for section in config._sections:
      config._sections[section] = OrderedDict(sorted(config._sections[section].items(), key=lambda t: t[0]))

    config._sections = OrderedDict(sorted(config._sections.items(), key=lambda t: t[0]))

  sio = io_compat.StringIO()
  config.write(sio)
  return sio.getvalue()


def _py2_read_properties(file, use_mock_default_section=True, config=None):
  if not os.path.isfile(file):
    raise ValueError("File does not exist: %s" % file)

  if config is None:
    config = cp.ConfigParser(allow_no_value=True)
    config.optionxform = str

  if use_mock_default_section:
    with open(file, 'r') as configfile:
      try:
        config.read_string('[DEFAULT]\n' + configfile.read())
      except:
        import StringIO

        config.readfp(StringIO.StringIO('[DEFAULT]\n' + configfile.read()))
  else:
    config.read(file)

  return config


if __name__ == "__main__":
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3
# pylint: disable=C0111
import os
import re
import subprocess
from collections import OrderedDict
from configparser import ConfigParser
from pathlib import Path
from typing import AnyStr, List, Sequence, Set, Tuple, Union, Pattern

import itertools

from ltpylib import filters


def replace_matches_in_file(file: Union[str, Path], search_string: str, replacement: str, quote_replacement: Union[bool, str] = False, force_replace: bool = False) -> bool:
  if isinstance(quote_replacement, str):
    quote_replacement = quote_replacement.lower() in ['true', '1', 't', 'y', 'yes']

  if quote_replacement:
    replacement = re.escape(replacement)

  content = read_file(file)
  content_new = re.sub(search_string, replacement, content)

  if content != content_new:
    write_file(file, content_new)
    return True
  elif force_replace and re.search(search_string, content):
    write_file(file, content_new)
    return True

  return False


def chmod_proc(perms: str, file: Union[str, Path]) -> int:
  if isinstance(file, str):
    file = Path(file)

  return subprocess.call(["chmod", perms, file.as_posix()])


def read_file(file: Union[str, Path]) -> AnyStr:
  if isinstance(file, str):
    file = Path(file)

  with open(file.as_posix(), 'r') as fr:
    content = fr.read()
  return content


def read_file_n_lines(file: Union[str, Path], n_lines: int) -> List[str]:
  if isinstance(file, str):
    file = Path(file)

  lines: List[str] = []
  with open(file.as_posix()) as fr:
    for n in range(n_lines - 1):
      line = fr.readline()
      if not line:
        break
      lines.append(line.rstrip('\n'))

  return lines


def write_file(file: Union[str, Path], contents: AnyStr):
  if isinstance(file, str):
    file = Path(file)

  with open(file.as_posix(), 'w') as fw:
    fw.write(contents)


def append_file(file: Union[str, Path], contents: AnyStr):
  if isinstance(file, str):
    file = Path(file)

  with open(file.as_posix(), 'a') as fw:
    fw.write(contents)


def list_files(dir: Path, globs: List[str] = ('**/*',)) -> List[Path]:
  files: Set[Path] = set()
  file: Path = None
  for file in list(itertools.chain(*[dir.glob(glob) for glob in globs])):
    if file.is_file():
      files.add(file)

  files_list = list(files)
  files_list.sort()
  return files_list


def list_dirs(base_dir: Path, globs: List[str] = ('**/*',)) -> List[Path]:
  dirs: Set[Path] = set()
  child_dir: Path = None
  for child_dir in list(itertools.chain(*[base_dir.glob(glob) for glob in globs])):
    if child_dir.is_dir():
      dirs.add(child_dir)

  dirs_list = list(dirs)
  dirs_list.sort()
  return dirs_list


def read_properties(file: Union[str, Path], use_mock_default_section: bool = True, config: ConfigParser = None) -> ConfigParser:
  if isinstance(file, str):
    file = Path(file)

  if not file.is_file():
    raise ValueError("File does not exist: %s" % file)

  if config is None:
    config = ConfigParser(allow_no_value=True)
    config.optionxform = str

  if use_mock_default_section:
    with open(file.as_posix(), 'r') as configfile:
      config.read_string('[DEFAULT]\n' + configfile.read())
  else:
    config.read(file.as_posix())

  return config


def write_properties(config: ConfigParser, file: Union[str, Path], sort_keys: bool = False):
  if sort_keys:
    if config._defaults:
      config._defaults = OrderedDict(sorted(config._defaults.items(), key=lambda t: t[0]))

    for section in config._sections:
      config._sections[section] = OrderedDict(sorted(config._sections[section].items(), key=lambda t: t[0]))

    config._sections = OrderedDict(sorted(config._sections.items(), key=lambda t: t[0]))

  if isinstance(file, str):
    file = Path(file)

  with open(file.as_posix(), 'w') as configfile:
    config.write(configfile)


def filter_files_with_matching_line(files: List[Union[str, Path]], regexes: List[Union[str, Pattern]], check_n_lines: int = 1) -> List[Path]:
  filtered: List[Path] = []
  for file in files:
    lines = read_file_n_lines(file, check_n_lines)
    has_match = False
    for line in lines:
      for regex in regexes:
        if re.search(regex, line):
          has_match = True
          break

      if has_match:
        break

    if not has_match:
      filtered.append(file)

  return filtered


def find_children(
    base_dir: Union[Path, str],
    break_after_match: bool = False,
    max_depth: int = -1,
    include_dirs: bool = True,
    include_files: bool = True,
    match_absolute_path: bool = False,
    include_patterns: Sequence[str] = None,
    exclude_patterns: Sequence[str] = None,
    includes: Sequence[str] = None,
    excludes: Sequence[str] = None,
    recursion_include_patterns: Sequence[str] = None,
    recursion_exclude_patterns: Sequence[str] = None,
    recursion_includes: Sequence[str] = None,
    recursion_excludes: Sequence[str] = None
) -> List[Path]:
  if isinstance(base_dir, str):
    top = str(base_dir)
  else:
    top = base_dir.as_posix()

  return _find_children(
    top,
    [],
    1,
    break_after_match,
    max_depth,
    include_dirs,
    include_files,
    match_absolute_path,
    include_patterns,
    exclude_patterns,
    includes,
    excludes,
    recursion_include_patterns,
    recursion_exclude_patterns,
    recursion_includes,
    recursion_excludes
  )[1]


def _find_children(
    top: str,
    found_dirs: List[Path],
    current_depth: int,
    break_after_match: bool,
    max_depth: int,
    include_dirs: bool,
    include_files: bool,
    match_absolute_path: bool,
    include_patterns: Sequence[str],
    exclude_patterns: Sequence[str],
    includes: Sequence[str],
    excludes: Sequence[str],
    recursion_include_patterns: Sequence[str],
    recursion_exclude_patterns: Sequence[str],
    recursion_includes: Sequence[str],
    recursion_excludes: Sequence[str]
) -> Tuple[bool, List[Path]]:
  found_match = False
  scandir_it = os.scandir(top)
  dirs = []

  with scandir_it:
    while True:
      try:
        try:
          entry = next(scandir_it)
        except StopIteration:
          break
      except OSError as error:
        return found_match, found_dirs

      try:
        is_dir = entry.is_dir()
      except OSError:
        # If is_dir() raises an OSError, consider that the entry is not
        # a directory, same behaviour than os.path.isdir().
        is_dir = False

      if is_dir and not include_dirs:
        continue
      elif not is_dir and not include_files:
        continue

      child = entry.name
      full_path = os.path.join(top, child)
      test_value = child if not match_absolute_path else full_path
      include = filters.should_include(
        test_value,
        include_patterns=include_patterns,
        exclude_patterns=exclude_patterns,
        includes=includes,
        excludes=excludes
      )

      if include:
        found_match = True
        found_dirs.append(Path(full_path))
        if break_after_match:
          break

      if is_dir:
        include_child = filters.should_include(
          test_value,
          include_patterns=recursion_include_patterns,
          exclude_patterns=recursion_exclude_patterns,
          includes=recursion_includes,
          excludes=recursion_excludes
        )
        if include_child:
          dirs.append(child)

  if (max_depth <= -1 or current_depth < max_depth) and (not found_match or not break_after_match):
    for dirname in dirs:
      _find_children(
        os.path.join(top, dirname),
        found_dirs,
        current_depth + 1,
        break_after_match,
        max_depth,
        include_dirs,
        include_files,
        match_absolute_path,
        include_patterns,
        exclude_patterns,
        includes,
        excludes,
        recursion_include_patterns,
        recursion_exclude_patterns,
        recursion_includes,
        recursion_excludes
      )

  return found_match, found_dirs


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
import re


def strip_color_codes(val: str) -> str:
  return re.sub(r"\x1B\[([0-9]{1,2}(;[0-9]{1,2})?)?[mGK]", "", val)


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
import datetime


def from_millis(millis: int) -> datetime.datetime:
  return datetime.datetime.fromtimestamp(millis / 1000.0)


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
import logging
import subprocess
import sys
from typing import Sequence

from ltpylib import procs


def confirm(question: str = 'Continue?', default: str = None) -> bool:
  """Ask a yes/no question via raw_input() and return their answer.

  "question" is a string that is presented to the user.
  "default" is the presumed answer if the user just hits <Enter>.
      It must be "yes" (the default), "no" or None (meaning
      an answer is required of the user).

  The "answer" return value is True for "yes" or False for "no".
  """
  valid = {
    "yes": True, "y": True, "ye": True,
    "no" : False, "n": False
  }
  if default is None:
    prompt = "\n[y/n]> "
  elif default == "yes" or default == "y":
    prompt = "\n[Y/n]> "
  elif default == "no" or default == "n":
    prompt = "\n[y/N]> "
  else:
    raise ValueError("invalid default answer: '%s'" % default)

  while True:
    choice = input(question + prompt).lower()
    if default is not None and choice == '':
      return valid[default]
    elif choice in valid:
      return valid[choice]
    else:
      logging.error("Please respond with 'yes' or 'no' "
                    "(or 'y' or 'n').\n")


def ask_for_input(prompt: str = 'Please input value.') -> str:
  return input(prompt + '\n> ')


def select_prompt(choices: Sequence[str], header: str = None, no_sort: bool = True, layout: str = "reverse") -> str:
  command = [
    'fzf',
    '--layout',
    layout,
  ]

  if header:
    command.extend(['--header', header])

  if no_sort:
    command.append('--no-sort')

  result: subprocess.CompletedProcess = procs.run(
    command,
    universal_newlines=True,
    stderr=sys.stderr,
    input="\n".join(choices)
  )

  if result.returncode == 130:
    raise KeyboardInterrupt

  result.check_returncode()
  return result.stdout.strip()


def select_prompt_old(choices: Sequence[str], message: str = None, bottom_message: str = None, max_width: int = None) -> str:
  command = ['select_prompt']

  if message:
    command.extend(['--message', message])

  if bottom_message:
    command.extend(['--bottom-message', message])

  if max_width:
    command.extend(['--max-width', str(max_width)])

  command.extend(choices)

  result: subprocess.CompletedProcess = procs.run(
    command,
    universal_newlines=True,
    stderr=sys.stderr,
  )
  if result.returncode == 130:
    raise KeyboardInterrupt

  result.check_returncode()
  return result.stdout.strip()


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
import collections
import logging
import subprocess
import sys
from typing import Any, List, Tuple

import psutil
from psutil import AccessDenied, NoSuchProcess, ZombieProcess
from time import sleep


def run(*popenargs, input=None, timeout=None, check=False, **kwargs) -> subprocess.CompletedProcess:
  kwargs['universal_newlines'] = True
  if 'stdout' not in kwargs:
    kwargs['stdout'] = subprocess.PIPE
  if 'stderr' not in kwargs:
    kwargs['stderr'] = subprocess.PIPE

  return subprocess.run(*popenargs, input=input, timeout=timeout, check=check, **kwargs)


def run_with_regular_stdout(*popenargs, input=None, timeout=None, check=False, **kwargs) -> subprocess.CompletedProcess:
  return run(*popenargs, input=input, timeout=timeout, check=check, stdout=sys.stdout, stderr=sys.stderr, **kwargs)


def run_and_parse_output(*popenargs, input=None, timeout=None, check=False, **kwargs) -> Tuple[int, str]:
  kwargs['stdout'] = subprocess.PIPE
  kwargs['universal_newlines'] = True
  if 'stderr' not in kwargs:
    kwargs['stderr'] = subprocess.PIPE

  result = subprocess.run(*popenargs, input=input, timeout=timeout, check=check, **kwargs)
  return result.returncode, result.stdout


def get_procs_from_name(name_matcher: str) -> List[Tuple[int, str]]:
  matched_procs = []
  exit_code, output = run_and_parse_output(['pgrep', '-fl', name_matcher])
  if exit_code > 0:
    return matched_procs

  for line in output.splitlines():
    parts = line.partition(' ')
    matched_procs.append((int(parts[0]), parts[2]))

  return matched_procs


def proc_debug_string(proc: psutil.Process) -> str:
  info = collections.OrderedDict()
  info['pid'] = proc.pid
  try:
    info["name"] = proc.name()
    if proc._create_time:
      info['started'] = psutil._pprint_secs(proc._create_time)
  except ZombieProcess:
    info["status"] = "zombie"
  except NoSuchProcess:
    info["status"] = "terminated"
  except AccessDenied:
    pass
  info['cmdline'] = ' '.join(proc.cmdline())
  return "%s.%s(%s)" % (
    proc.__class__.__module__,
    proc.__class__.__name__,
    ", ".join(["%s=%r" % (k, v) for k, v in info.items()]),
  )


def stop_proc_by_name(name_matcher: str) -> bool:
  matched_procs = get_procs_from_name(name_matcher)
  if not matched_procs:
    return False

  if len(matched_procs) > 1:
    raise Exception('Multiple processes found: %s' % matched_procs)

  proc = psutil.Process(matched_procs[0][0])
  proc.terminate()
  return True


def stop_proc_by_pid(pid: Any) -> bool:
  proc = psutil.Process(int(pid))
  if proc.is_running():
    proc.terminate()
  else:
    return False
  return True


def await_termination(pid: int, timeout: int = 30, sleep_time: int = 1, log_level: int = logging.INFO):
  if pid is None:
    return

  proc = psutil.Process(pid)
  total_time = 0
  while total_time <= timeout:
    if proc.is_running():
      logging.log(log_level, 'STATUS: loop %s seconds - process is still running, please wait...', total_time)
    else:
      break

    total_time += sleep_time
    sleep(sleep_time)

  if proc.is_running():
    logging.error('STATUS: Process still running after %s seconds, sending SIGKILL to terminate...', total_time)
    proc.kill()
    proc.wait(5)
    logging.log(log_level, 'STATUS: Process killed.')
  else:
    logging.log(log_level, 'STATUS: Process successfully shutdown after %s seconds.', total_time)


if __name__ == "__main__":
  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3
import os
import tempfile

from ltpylib import files, procs


def diff_git(initial: str, updated: str, add_suffix: str = None):
  suffix_prefix = ('.' + add_suffix) if add_suffix else ''

  ifd, initial_temp_file = tempfile.mkstemp(suffix=suffix_prefix + '.initial')
  ufd, updated_temp_file = tempfile.mkstemp(suffix=suffix_prefix + '.updated')
  os.close(ifd)
  os.close(ufd)

  files.write_file(initial_temp_file, initial)
  files.write_file(updated_temp_file, updated)

  result = procs.run(
    [
      'git',
      'diff',
      '--color=always',
      '--no-index',
      '--color-words',
      '-w',
      initial_temp_file,
      updated_temp_file
    ],
    check=False
  )

  os.remove(initial_temp_file)
  os.remove(updated_temp_file)

  return result.stdout



#!/usr/bin/env python3

import time


def format_seconds(frac_seconds):
  hours, rem = divmod(float(frac_seconds), 3600)
  minutes, seconds = divmod(rem, 60)
  return "{0:0>2}:{1:0>2}:{2:06.3f}".format(int(hours), int(minutes), seconds)


def format_millis(millis):
  return format_seconds(float(millis) / float(1000))


def get_time_elapsed_msg(start_time):
  return format_seconds(time.time() - float(start_time))


def get_time_remaining_msg(start_time, count, total):
  frac_seconds = time.time() - float(start_time)
  if float(count) <= 0:
    return "Elapsed: {0: >12} Remaining: {1: >12}".format(format_seconds(frac_seconds), 'N/A')

  estimated_total = frac_seconds * (float(total) / float(count))
  return "Elapsed: {0: >12} Remaining: {1: >12}".format(format_seconds(frac_seconds), format_seconds(estimated_total - frac_seconds))


if __name__ == "__main__":
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3
import argparse
import logging
import os
import shutil
import subprocess
import sys
from pathlib import Path

DEFAULT_LOG_LEVEL = 'INFO'
DEFAULT_LOG_FORMAT = '{message}'
DEFAULT_LOG_STYLE = '{'
LOG_FORMAT_WITH_LEVEL = '{levelname:<8} {message}'
LOG_SEP = '------------------------------------------------'


class StdoutStreamHandler(logging.StreamHandler):

  def __init__(self):
    super().__init__(stream=sys.stdout)

  def handleError(self, record):
    err_type, err_value, traceback = sys.exc_info()
    if err_type == BrokenPipeError:
      exit(0)

    super().handleError(record)


def init_logging(verbose: bool = None, log_level: str = None, log_format: str = None, args: argparse.Namespace = None):
  if args:
    if verbose is None and 'verbose' in args:
      verbose = args.verbose
    if log_level is None and 'log_level' in args:
      log_level = args.log_level
    if log_format is None and 'log_format' in args:
      log_format = args.log_format

  if verbose:
    log_level = 'DEBUG'
  elif log_level is not None:
    log_level = log_level
  elif os.environ.get('log_level') is not None:
    log_level = os.environ.get('log_level')
  else:
    log_level = DEFAULT_LOG_LEVEL

  logging.basicConfig(
    level=log_level, style=DEFAULT_LOG_STYLE, format=(log_format or DEFAULT_LOG_FORMAT), handlers=[StdoutStreamHandler()])


def is_debug_enabled():
  return logging.root.isEnabledFor(logging.DEBUG)


def log_sep(debug_only=False):
  if debug_only:
    logging.debug(LOG_SEP)
  else:
    logging.info(LOG_SEP)


def create_path_log_info(path: Path, replace_home_dir: bool = True) -> str:
  output = path.as_posix()

  if replace_home_dir:
    output = output.replace(os.getenv('HOME'), '~', 1)

  return output


def log_with_sep(level: int, msg, *args, **kwargs):
  logging.log(level, LOG_SEP)
  logging.log(level, msg, *args, **kwargs)
  logging.log(level, LOG_SEP)


def log_with_title_sep(level: int, title, msg, *args, **kwargs):
  logging.log(level, title)
  logging.log(level, LOG_SEP)
  logging.log(level, msg, *args, **kwargs)
  logging.log(level, '')


def tail_log_file(file: str, *func_args):
  if shutil.which('multitail'):
    log_cmd = ['multitail']
    mt_conf = Path(os.getenv('DOTFILES') + '/multitail.conf')
    if '-F' not in func_args and mt_conf.is_file():
      log_cmd.extend(['-F', mt_conf.as_posix()])

    if '-CS' not in func_args:
      log_cmd.extend(['-CS', 'l4j'])

    if '-n' not in func_args:
      log_cmd.extend(['-n', '1500'])

  else:
    log_cmd = ['tail', '-1500f']

  if func_args:
    log_cmd.extend(func_args)

  log_cmd.append(file)

  subprocess.check_call(log_cmd, universal_newlines=True)


if __name__ == "__main__":
  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3

import logging
import re
from pathlib import Path
from typing import List, Sequence, Pattern

from ltpylib import files, logs

SKIP_FILE_REGEX: Pattern = re.compile(r"^[^a-zA-Z0-9]*template:skip\s*$")
START_REGEX = re.compile(r"^[^a-zA-Z0-9]*template:start (.*?)$")
END_REGEX = re.compile(r"^[^a-zA-Z0-9]*template:end\s*$")
DEFAULT_CANDIDATE_FILES_GLOBS: Sequence[str] = ('**/*',)
DEFAULT_TEMPLATE_FILES_GLOBS: Sequence[str] = ('**/*.template.*',)


def replace_templates_in_files(
    candidate_files_dir: Path,
    template_dir: Path,
    candidate_files_globs: List[str] = DEFAULT_CANDIDATE_FILES_GLOBS,
    template_files_globs: List[str] = DEFAULT_TEMPLATE_FILES_GLOBS,
    debug_mode: bool = False,
    force_replace: bool = False
) -> bool:
  candidate_files: List[Path] = files.filter_files_with_matching_line(
    files.list_files(candidate_files_dir, candidate_files_globs),
    [SKIP_FILE_REGEX],
    check_n_lines=2
  )
  if not candidate_files:
    logging.warning("No candidate files found: candidate_files_dir=%s candidate_files_globs=%s", candidate_files_dir.as_posix(), candidate_files_globs)
    return False

  logs.log_with_title_sep(logging.DEBUG, 'Candidate Files', '\n'.join([file.absolute().as_posix() for file in candidate_files]))

  template_files: List[Path] = files.list_files(template_dir, template_files_globs)
  if not template_files:
    logging.warning("No template files found: template_dir=%s template_files_globs=%s", template_dir.as_posix(), template_files_globs)
    return False

  logs.log_with_title_sep(logging.DEBUG, 'Template Files', '\n'.join([file.absolute().as_posix() for file in template_files]))

  for template_file in template_files:
    replace_template(candidate_files, template_file, debug_mode=debug_mode, force_replace=force_replace)

  return True


def replace_template(candidate_files: List[Path], template_file: Path, debug_mode: bool = False, force_replace: bool = False):
  template_file_content: str = files.read_file(template_file)

  lines = []
  found_start = False
  search_string: str = ''

  for line in template_file_content.splitlines():
    if found_start:
      if END_REGEX.fullmatch(line):
        break

      lines.append(line)
    elif START_REGEX.fullmatch(line):
      found_start = True
      search_string = START_REGEX.fullmatch(line).group(1)

  if not lines or not search_string:
    logging.error('Something went wrong, skipping template: template_file=%s lines_count=%s search_string=%s', template_file.as_posix(), len(lines), search_string)
    return

  replacement: str = '\n'.join(lines)

  logs.log_with_title_sep(logging.DEBUG, 'template_file', template_file.as_posix())
  logs.log_with_title_sep(logging.DEBUG, 'search_string', search_string)
  logs.log_with_title_sep(logging.DEBUG, 'replacement', replacement)

  if not debug_mode:
    for file in candidate_files:
      if file.absolute().as_posix() == template_file.absolute().as_posix():
        continue

      if files.replace_matches_in_file(file, search_string, replacement, quote_replacement=False, force_replace=force_replace):
        logging.info("Replaced template: %s -> %s", template_file.as_posix(), file.as_posix())


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)



#!/usr/bin/env python3
# pylint: disable=C0111
from typing import List


def find(key: str, obj: dict) -> List[dict]:
  if isinstance(obj, dict):
    for k, v in obj.items():
      if k == key:
        yield v
      else:
        for res in find(key, v):
          yield res
  elif isinstance(obj, list):
    for d in obj:
      for res in find(key, d):
        yield res

  # for k, v in obj.items():
  #   if k == key:
  #     yield v
  #   elif isinstance(v, dict):
  #     for res in find(key, v):
  #       yield res
  #   elif isinstance(v, list):
  #     for d in v:
  #       for res in find(key, d):
  #         yield res


if __name__ == "__main__":
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)



#!/usr/bin/env python3

import argparse
import itertools
import os
import select
import sys
from typing import List, Optional, Sequence

import argcomplete

from ltpylib import logs

TRUE_VALUES = ['true', '1', 't', 'yes', 'y']
DEFAULT_POSITIONALS_KEY = 'command'


class PositionalsHelpFormatter(argparse.HelpFormatter):

  def __init__(self,
               prog,
               indent_increment=2,
               max_help_position=24,
               width=None,
               positionals_key: str = DEFAULT_POSITIONALS_KEY):
    super().__init__(prog, indent_increment, max_help_position, width)
    self.positionals_key = positionals_key
    self.positionals_action = argparse._StoreAction([], self.positionals_key, nargs='+')

  def add_usage(self, usage, actions, groups, prefix=None):
    super().add_usage(usage, itertools.chain(actions, [self.positionals_action]), groups, prefix)

  def add_arguments(self, actions):
    if self._current_section.heading == 'positional arguments':
      super().add_arguments(itertools.chain(actions, [self.positionals_action]))
    else:
      super().add_arguments(actions)


def add_default_arguments_to_parser(arg_parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
  arg_parser.add_argument('-v', '--verbose', action='store_true')
  arg_parser.add_argument('--dry-run', action='store_true')
  return arg_parser


def check_debug_mode() -> bool:
  return os.getenv('debug_mode', 'false').lower() in TRUE_VALUES


def check_verbose() -> bool:
  return os.getenv('verbose', 'false').lower() in TRUE_VALUES


def create_default_arg_parser() -> argparse.ArgumentParser:
  arg_parser = argparse.ArgumentParser()
  return add_default_arguments_to_parser(arg_parser)


def create_default_with_positionals_arg_parser() -> argparse.ArgumentParser:
  arg_parser = argparse.ArgumentParser(formatter_class=PositionalsHelpFormatter)
  return add_default_arguments_to_parser(arg_parser)


def parse_args(arg_parser: argparse.ArgumentParser, argv: Optional[Sequence[str]] = None) -> argparse.Namespace:
  argcomplete.autocomplete(arg_parser)
  args = arg_parser.parse_args(args=argv)
  return args


def parse_args_and_init_others(arg_parser: argparse.ArgumentParser, argv: Optional[Sequence[str]] = None) -> argparse.Namespace:
  args = parse_args(arg_parser, argv=argv)
  logs.init_logging(args=args)
  return args


def parse_args_with_positionals_and_init_others(arg_parser: argparse.ArgumentParser, positionals_key: str = DEFAULT_POSITIONALS_KEY,
                                                argv: Optional[Sequence[str]] = None) -> argparse.Namespace:
  argcomplete.autocomplete(arg_parser)
  args, positionals = arg_parser.parse_known_intermixed_args(args=argv)
  args.__setattr__(positionals_key, positionals)
  logs.init_logging(args=args)
  return args


def does_stdin_have_data() -> bool:
  if select.select([sys.stdin, ], [], [], 0.0)[0]:
    return True
  else:
    return False


class BaseArgs(object):

  def __init__(self, args: argparse.Namespace):
    self._args: argparse.Namespace = args
    self.verbose: bool = args.verbose
    self.dry_run: bool = args.dry_run


class IncludeExcludeCmdArgs(object):

  def __init__(self, args: argparse.Namespace):
    self.exclude_commands: List[str] = args.exclude_cmd or []
    self.include_commands: List[str] = args.include_cmd or []

  @staticmethod
  def add_arguments_to_parser(arg_parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    arg_parser.add_argument('--exclude-cmd', action='append')
    arg_parser.add_argument('--include-cmd', action='append')
    return arg_parser


class IncludeExcludeRegexArgs(object):

  def __init__(self, args: argparse.Namespace):
    self.exclude_regex: List[str] = args.exclude_regex or []
    self.include_regex: List[str] = args.include_regex or []

  @staticmethod
  def add_arguments_to_parser(arg_parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    arg_parser.add_argument('--exclude-regex', action='append')
    arg_parser.add_argument('--include-regex', action='append')
    return arg_parser



#!/usr/bin/env python3
from typing import List, Tuple

import stashy
from requests import Response
from stashy import errors, helpers
from stashy.pullrequests import PullRequest, PullRequests
from stashy.repos import Repos

from ltpylib.stash_types import PullRequestMergeStatus, PullRequestMergeability, PullRequestStatus, Repository

STASH_URL: str = "https://stash.wlth.fr"


class StashApi(object):

  def __init__(self, stash: stashy.Stash):
    self.stash: stashy.Stash = stash

  def repos_for_project(self, project: str) -> List[Repository]:
    repos: Repos = self.stash.projects[project].repos
    return [
      Repository(repo) for repo in repos.list()
    ]

  def create_pull_request(
      self,
      project: str,
      repo: str,
      from_ref: str,
      to_ref: str,
      title: str,
      description: str,
      reviewers: List[str]
  ) -> PullRequestStatus:
    prs: PullRequests = self.stash.projects[project].repos[repo].pull_requests
    response: dict = prs.create(
      title=title,
      fromRef=from_ref,
      toRef=to_ref,
      description=description,
      reviewers=reviewers
    )

    return PullRequestStatus(response)

  def delete_pull_request_source_branch(
      self,
      project: str,
      repo: str,
      id: int,
  ) -> Response:
    delete_source_branch_url = '%s/rest/pull-request-cleanup/latest/projects/%s/repos/%s/pull-requests/%s' % (
      self.stash._client._base_url,
      project,
      repo,
      str(id)
    )
    kw = helpers.add_json_headers({})
    delete_source_branch_response = self.stash._client._session.post(
      delete_source_branch_url,
      json={"deleteSourceRef": True, "retargetDependents": True},
      **kw
    )
    return delete_source_branch_response

  def merge_pull_request(
      self,
      project: str,
      repo: str,
      id: int,
      delete_source_branch: bool = True,
      commit_message: str = "",
      version: int = None
  ) -> PullRequestMergeStatus:
    pr: PullRequest = self.stash.projects[project].repos[repo].pull_requests[str(id)]
    if version is None:
      pr_info = self.pull_request(project, repo, id, include_merge_info=False)
      version = pr_info.version

    result: PullRequestMergeStatus = PullRequestMergeStatus(self._parse_raw_response(pr._client.post(pr.url("/merge"), data=dict(message=commit_message, version=version))))
    if delete_source_branch and result.state == "MERGED":
      delete_source_branch_response = self.delete_pull_request_source_branch(project, repo, id)
      result.sourceBranchDeleted = delete_source_branch_response.status_code == 200 or delete_source_branch_response.status_code == 204

    return result

  def pull_request(
      self,
      project: str,
      repo: str,
      id: int,
      include_merge_info: bool = True
  ) -> PullRequestStatus:
    pr: PullRequest = self.stash.projects[project].repos[repo].pull_requests[str(id)]
    result: PullRequestStatus = PullRequestStatus(pr.get())
    if include_merge_info and result.open:
      merge_info = self.pull_request_merge_info(project, repo, id)
      result.mergeInfo = merge_info

    return result

  def pull_request_merge_info(
      self,
      project: str,
      repo: str,
      id: int,
  ) -> PullRequestMergeability:
    pr: PullRequest = self.stash.projects[project].repos[repo].pull_requests[str(id)]
    return PullRequestMergeability(pr.merge_info())

  def pull_requests(
      self,
      project: str,
      repo: str,
      direction: str = 'INCOMING',
      at: str = None,
      state: str = 'OPEN',
      order: str = None,
      author: str = None
  ) -> List[PullRequestStatus]:
    prs: PullRequests = self.stash.projects[project].repos[repo].pull_requests
    return [
      PullRequestStatus(pr) for pr in prs.all(
        direction=direction,
        at=at,
        state=state,
        order=order,
        author=author,
      )
    ]

  def _parse_raw_response(self, response: Response) -> dict:
    errors.maybe_throw(response)
    try:
      return response.json()
    except ValueError:
      return response.text


def create_stash_api(url: str, creds: Tuple[str, str]) -> StashApi:
  return StashApi(stashy.connect(url, creds[0], creds[1]))



#!/usr/bin/env python3
# pylint: disable=C0103

from setuptools import setup, find_packages

requirements = open('./requirements.txt').read().splitlines()
long_description = open('./README.md').read()

setup(
  name='ltpylib',
  version='0.0.2',
  description='Common Python helper functions',
  long_description=long_description,
  long_description_content_type='text/markdown',
  url='https://github.com/lancethomps/ltpylib',
  project_urls={
      'Bug Reports': 'https://github.com/lancethomps/ltpylib/issues',
      'Source': 'https://github.com/lancethomps/ltpylib',
  },
  author='Lance Thompson',
  license='MIT',
  keywords='utils',

  python_requires='>=3',
  extras_require={
      'dev': ['check-manifest'],
      'test': ['coverage'],
  },
  packages=find_packages(exclude=['contrib', 'docs', 'scripts', 'tests']),
  install_requires=requirements,
  classifiers=[],
)



#!/usr/bin/env python3
import logging
import os
import sys
from getpass import getpass, getuser

from ltpylib import inputs, procs


def notify(message: str, title: str = "Terminal Notification", sound_name: str = "Ping", subtitle: str = ""):
  message = message.replace('"', '\\"')
  title = title.replace('"', '\\"')
  sound_name = sound_name.replace('"', '\\"')
  subtitle = subtitle.replace('"', '\\"')

  js_function = f"""
var app = Application.currentApplication()
 
app.includeStandardAdditions = true
 
app.displayNotification("{message}", {{
    withTitle: "{title}",
    subtitle: "{subtitle}",
    soundName: "{sound_name}"
}})
"""
  result = procs.run_with_regular_stdout([
    "/usr/bin/osascript",
    "-l",
    "JavaScript",
    "-e",
    js_function
  ])
  if __name__ == "__main__":
    exit(result.returncode)


def add_generic_password(label: str, pw: str, account: str = None) -> bool:
  if account is None:
    account = getuser()

  result = procs.run_with_regular_stdout([
    "security",
    "add-generic-password",
    "-a",
    account,
    "-s",
    label,
    "-w",
    pw
  ])
  if result.returncode != 0:
    raise Exception("Could not add generic keychain password: label=%s account=%s" % (label, account))

  return True


def find_generic_password(label: str, ask_if_missing: bool = True, add_if_missing: bool = False, prompt_to_add_if_missing: bool = True) -> str:
  status, pw = procs.run_and_parse_output([
    "security",
    "find-generic-password",
    "-ws",
    label,
  ])
  if status != 0:
    if ask_if_missing:
      prompt = 'Please enter your password for %s.' % (label)
      if not add_if_missing:
        prompt += ' You can skip entering your password by running: security add-generic-password -a "$USER" -s "%s" -w' % (label)

      pw = getpass(prompt="%s\n> " % prompt)
      if add_if_missing and pw:
        if not prompt_to_add_if_missing or inputs.confirm('Add this password to your keychain in order to skip this prompt next time?', default="y"):
          logging.info('Adding generic password to keychain...')
          add_generic_password(label, pw, account=getuser())
    else:
      raise Exception("Could not find generic keychain password for label: %s" % label)

  return pw.strip()


def _main():
  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  _main()



#!/usr/bin/env python3
# pylint: disable=C0111

import re
import urllib.parse
from typing import Callable, List

StrConverter = Callable[[str], str]

SEARCH_SEP_CHARS: List[str] = [
  " ",
  "_",
  "-",
  ".",
]


def stripper(val: str) -> str:
  return val.strip()


def lower_case(val: str) -> str:
  return val.lower()


def alphanum_only(val: str) -> str:
  return re.sub(r'[^a-zA-Z0-9]', '', val)


def url_encode(val: str) -> str:
  return urllib.parse.quote_plus(val)


def url_decode(val: str) -> str:
  return urllib.parse.unquote_plus(val)


def to_camel_case(val: str, sep: str = None) -> str:
  if not val:
    return val

  if sep is None:
    if val.count(" ") > 0:
      sep = " "
    else:
      count: int = -1
      for sep_char in SEARCH_SEP_CHARS:
        sep_char_count = val.count(sep_char)
        if sep_char_count > count:
          count = sep_char_count
          sep = sep_char

  val = val.lower().replace(sep, " ").title()
  return (val[0:1].lower() + val[1:]).replace(" ", "")


def _main():
  import sys

  result = globals()[sys.argv[1]](*sys.argv[2:])
  if result is not None:
    print(result)


if __name__ == "__main__":
  try:
    _main()
  except KeyboardInterrupt:
    exit(130)
